
# https://mkdocstrings.github.io/recipes/

from pathlib import Path

import mkdocs_gen_files

nav = mkdocs_gen_files.Nav()

for path in sorted(Path("src").rglob("*.py")):

    module_path = path.relative_to("src").with_suffix("")

    doc_path = path.relative_to("src").with_suffix(".md")

    full_doc_path = Path("reference", doc_path)

    parts = list(module_path.parts)
   
    if parts[-1] == "__init__":
        parts = parts[:-1]
        print("init parts:", parts)
    if parts[-1] in ["notebooks", "settings", "restful_config", "__main__"]:
        continue

    nav[parts] = doc_path.as_posix() 

    with mkdocs_gen_files.open(full_doc_path, "w") as fd:

        identifier = ".".join(parts)
        if "calcat_interface" == full_doc_path.stem:
            print(
"""
??? Note "This module help in skipping calibrationDBRemote"
    ![Calibration DB Remote avoided](../xfel-calibrate_cli_process_no_caldbremote.png)
""",
                file=fd)
        print("::: " + "src." + identifier, file=fd)

    mkdocs_gen_files.set_edit_path(full_doc_path, path)

with mkdocs_gen_files.open(f"reference/summary.md", "w") as nav_file:
    nav_file.writelines(nav.build_literate_nav())
